@@ -99,7 +99,8 @@ class TestApp(object):
         .. versionadded:: 2.0
 
         It can also be an actual full URL to an http server and webtest
-        will proxy requests with `wsgiproxy`.
+        will proxy requests with `WSGIProxy2
+        <https://pypi.python.org/pypi/WSGIProxy2/>`_.
     :type app:
         WSGI application
     :param extra_environ:
@@ -195,6 +196,7 @@ class TestApp(object):
         if value is not None:
             invalid_value = (
                 "You should use a value like ('Basic', ('user', 'password'))"
+                " OR ('Bearer', 'token') OR ('JWT', 'token')"
             )
             if isinstance(value, (list, tuple)) and len(value) == 2:
                 authtype, val = value
@@ -203,6 +205,9 @@ class TestApp(object):
                     val = ':'.join(list(val))
                     val = b64encode(to_bytes(val)).strip()
                     val = val.decode('latin1')
+                elif authtype in ('Bearer', 'JWT') and val and \
+                        isinstance(val, (str, text_type)):
+                    val = val.strip()
                 else:
                     raise ValueError(invalid_value)
                 value = str('%s %s' % (authtype, val))
@@ -226,6 +231,10 @@ class TestApp(object):
         Sets a cookie to be passed through with requests.
 
         """
+        cookie_domain = self.extra_environ.get('HTTP_HOST', '.localhost')
+        cookie_domain = cookie_domain.split(':', 1)[0]
+        if '.' not in cookie_domain:
+            cookie_domain = "%s.local" % cookie_domain
         value = escape_cookie_value(value)
         cookie = http_cookiejar.Cookie(
             version=0,
@@ -233,7 +242,7 @@ class TestApp(object):
             value=value,
             port=None,
             port_specified=False,
-            domain='.localhost',
+            domain=cookie_domain,
             domain_specified=True,
             domain_initial_dot=False,
             path='/',
@@ -498,6 +507,10 @@ class TestApp(object):
             if isinstance(value, forms.File):
                 if value.value:
                     _append_file([key] + list(value.value))
+                else:
+                    # If no file was uploaded simulate an empty file with no
+                    # name like real browsers do:
+                    _append_file([key, b'', b''])
             elif isinstance(value, forms.Upload):
                 file_info = [key, value.filename]
                 if value.content is not None:
@@ -506,8 +519,15 @@ class TestApp(object):
                         file_info.append(value.content_type)
                 _append_file(file_info)
             else:
-                if isinstance(value, text_type):
+                if isinstance(value, int):
+                    value = str(value).encode('utf8')
+                elif isinstance(value, text_type):
                     value = value.encode('utf8')
+                elif not isinstance(value, (bytes, str)):
+                    raise ValueError((
+                        'Value for field {0} is a {1} ({2}). '
+                        'It must be str, bytes or an int'
+                    ).format(key, type(value), value))
                 lines.extend([
                     b'--' + boundary,
                     b'Content-Disposition: form-data; name="' + key + b'"',
@@ -601,7 +621,7 @@ class TestApp(object):
         # verify wsgi compatibility
         app = lint.middleware(self.app) if self.lint else self.app
 
-        ## FIXME: should it be an option to not catch exc_info?
+        # FIXME: should it be an option to not catch exc_info?
         res = req.get_response(app, catch_exc_info=True)
 
         # be sure to decode the content
@@ -613,7 +633,7 @@ class TestApp(object):
         res.app = app
         res.test_app = self
 
-        # We do this to make sure the app_iter is exausted:
+        # We do this to make sure the app_iter is exhausted:
         try:
             res.body
         except TypeError:  # pragma: no cover
@@ -663,7 +683,7 @@ class TestApp(object):
                 res)
         if status != res.status_int:
             raise AppError(
-                "Bad response: %s (not %s)", res_status, status)
+                "Bad response: %s (not %s)\n%s", res_status, status, res)
 
     def _check_errors(self, res):
         errors = res.errors
